#
# Copyright (c) 2003, Theodore A. Roth  <troth@openavr.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
# * Neither the name of the copyright holders nor the names of
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# $Id$
#

##
## Either copy this file to Makefile or have your Makefile include this file.
## In either case, you need to set the following variables in Makefile:
##
##    PRG        : [required] The name of the project.
##    OBJ        : [required] List of .o object files making up the project.
##    AVR_TARGET : [required] Target microcontroller (e.g. atmega128)
##    OPTIMIZE   : [optional] Optimization level (e.g. -O2)
##    DEFS       : [optional] Compile time options (e.g. -D_FOO_ -I/tmp)
##    LIBS       : [optional] Any libraries to link against (e.g. -lm)
##

CC               = avr-gcc
OBJCOPY          = avr-objcopy
OBJDUMP          = avr-objdump

CFLAGS           = -g $(OPTIMIZE) -Wall -mmcu=$(AVR_TARGET) $(DEFS)
LDFLAGS          = -Wl,-Map,$(PRG).map

# The default rule.

all: elf lst hex bin srec eeprom

elf: $(PRG).elf

$(PRG).elf: $(OBJ)
	$(CC) $(CFLAGS) $(LDFLAGS) -o $@ $^ $(LIBS)

# Generates the .lst file (disassemble of object file).

lst: $(PRG).lst
%.lst: %.elf
	$(OBJDUMP) -h -S -d --disassemble-zeroes $< > $@

# Generates an Intel hex object file from the elf object.

hex: $(PRG).hex
%.hex: %.elf
	$(OBJCOPY) -j .text -j .data -O ihex $< $@

# Generates a Motorola S-Record object file from the elf object.

srec: $(PRG).srec
%.srec: %.elf
	$(OBJCOPY) -j .text -j .data -O srec $< $@

# Generates a raw binary object file from the elf object.

bin: $(PRG).bin
%.bin: %.elf
	$(OBJCOPY) -j .text -j .data -O binary $< $@

# Rules for building the .eeprom rom images

eeprom: ehex ebin esrec

ehex:  $(PRG)_eeprom.hex
%_eeprom.hex: %.elf
	$(OBJCOPY) -j .eeprom --change-section-lma .eeprom=0 -O ihex $< $@

esrec: $(PRG)_eeprom.srec
%_eeprom.srec: %.elf
	$(OBJCOPY) -j .eeprom --change-section-lma .eeprom=0 -O srec $< $@

ebin:  $(PRG)_eeprom.bin
%_eeprom.bin: %.elf
	$(OBJCOPY) -j .eeprom --change-section-lma .eeprom=0 -O binary $< $@

# Cleanup

clean:
	rm -rf $(PRG) *.o *.elf *.lst *.bin *.hex *.srec core *.core *.map
	rm -rf *.i *.s $(EXTRA_CLEAN_FILES)

# Variables and rules for uploading programs to the target device.

UISP_OPTS   = -dprog=stk500 -dpart=$(AVR_TARGET)

UISP = uisp

erase:
	$(UISP) $(UISP_OPTS) --erase

upload: $(PRG).srec
	$(UISP) $(UISP_OPTS) --erase --upload if=$(PRG).srec

verify: $(PRG).srec
	$(UISP) $(UISP_OPTS) --verify if=$(PRG).srec

download:
	$(UISP) $(UISP_OPTS) --download

.PHONY: erase upload verify download
