/* Copyright (c) 2003, Theodore A. Roth
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */

/* Contributed by: Theodore A. Roth  <troth@openavr.org> */

/* $Id$ */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include <avr/sleep.h>

/* The mega169 uses TIMSK0 instead of TIMSK and TCCR0A instead of TCCR0 */

#if !defined(TIMSK) && defined(TIMSK0)
#  define TIMSK TIMSK0
#endif

#if !defined(TCCR0) && defined(TCCR0A)
#  define TCCR0 TCCR0A
#endif

static uint8_t cnt;

void
init_port (void)
{
#if defined(PORTB)
    PORTB = 0xff;
    DDRB = 0xff;
#endif
}

void
out_port (void)
{
#if defined(PORTB)
        PORTB = cnt;
#endif
}

void
init_timer (void)
{
    /* Zero out the counter */
    TCNT0 = 0;

    /* Enable the Timer Overflow Interrupt */
    TIMSK |= _BV (TOIE0);

    /* Configure and start the timer/counter. */
    TCCR0 = _BV (CS00) | _BV (CS01) | _BV (CS02) ;

    /* enable global interrupts */
    sei ();
}

SIGNAL (SIG_OVERFLOW0)
{
    /* Does nothing but clear the interrupt. */

    cnt++;
}

int
main (void)
{
    init_port ();
    init_timer ();

    set_sleep_mode (SLEEP_MODE_IDLE);

    while (1)
    {
        sleep_mode ();
        out_port ();
    }

    return(0);
}

